@@ -1,6 +1,9 @@
 //
 // /home/ms/source/sidplay/libsidplay/emu/RCS/6581_.cpp,v
 //
+// --------------------------------------------------------------------------
+// Copyright (c) 1994-1997 Michael Schwendt. All rights reserved.
+//
 // Contributions:
 //
 // Noise generation algorithm is used courtesy of Asger Alstrup Nielsen.
@@ -10,21 +13,20 @@
 // each table does not differ.
 //
 // MOS-8580 R5 combined waveforms recorded by Dennis "Deadman" Lindroos.
-// --------------------------------------------------------------------------
 //
-// --- MOS-6581 Emulator ---
+//  This program is free software; you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation; either version 2 of the License, or
+//  (at your option) any later version.
 //
-// Copyright (c) 1994-1997 Michael Schwendt. All rights reserved.
+//  This program is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//  GNU General Public License for more details.
 //
-// Redistribution and use  in source and  binary forms, either  unchanged or
-// modified, are permitted provided that the following conditions are met:
-//
-// (1)  Redistributions  of  source  code  must  retain  the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// (2) Redistributions  in binary  form must  reproduce the  above copyright
-// notice,  this  list  of  conditions  and  the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
+//  You should have received a copy of the GNU General Public License
+//  along with this program; if not, write to the Free Software
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // THIS SOFTWARE  IS PROVIDED  BY THE  AUTHOR ``AS  IS'' AND  ANY EXPRESS OR
 // IMPLIED  WARRANTIES,  INCLUDING,   BUT  NOT  LIMITED   TO,  THE   IMPLIED
@@ -91,7 +93,7 @@ static ubyte* waveform70;
 #endif
 
 static const udword noiseSeed = 0x7ffff8;
-uword PCMfreq;
+udword PCMfreq;
 static udword PCMsid, PCMsidNoise;
 
 // Song clock speed (PAL or NTSC). Does not affect pitch.
@@ -193,7 +195,7 @@ void sidEmuSetReplayingSpeed(int clockMode, uword callsPerSec)
 			{
 				timer = defaultTimer;
 			}
-			calls = sidtuneClockSpeed / timer;
+			calls = ( ((sidtuneClockSpeed<<1)/timer) +1) >>1;
 			break;
 		}
 	 default:
@@ -214,7 +216,7 @@ static void sidEmuUpdateReplayingSpeed()
 		// Prevent overflow
 		if ( timer < 16 )
 			timer = defaultTimer;
-		calls = sidtuneClockSpeed / timer;
+		calls = ( ((sidtuneClockSpeed<<1)/timer) +1) >>1;
 		calcValuesPerCall();
 	}
 }
@@ -890,7 +892,6 @@ ubyte playRamRom;
 #if defined(SIDEMU_TIME_COUNT)
 static udword prevBufferLen;    // need for fast_forward time count
 static udword scaledBufferLen;
-udword bytesCountSong;
 #endif
 
 void* fill8bitMono(void*, udword); // only need one fill()-prototype here
@@ -923,16 +924,16 @@ void sidEmuFillBuffer( emuEngine& thisEmu,
 			prevBufferLen = bufferLen;
 			scaledBufferLen = (bufferLen<<7) / fastForwardFactor;
 		}
-		bytesCountSong += scaledBufferLen;
-        thisEmu.bytesCount += bufferLen;
-		while (thisEmu.bytesCount >= thisEmu.config.frequency)
+		thisEmu.bytesCountTotal += bufferLen;
+        thisEmu.bytesCountSong += scaledBufferLen;
+		while (thisEmu.bytesCountTotal >= thisEmu.config.frequency)
 		{
-			thisEmu.bytesCount -= thisEmu.config.frequency;
+			thisEmu.bytesCountTotal -= thisEmu.config.frequency;
 			thisEmu.secondsTotal++;
 		}
-		while (bytesCountSong >= thisEmu.config.frequency)
+		while (thisEmu.bytesCountSong >= thisEmu.config.frequency)
 		{
-			bytesCountSong -= thisEmu.config.frequency;
+			thisEmu.bytesCountSong -= thisEmu.config.frequency;
 			thisEmu.secondsThisSong++;
 		}
 #endif
@@ -1190,19 +1191,25 @@ void initWaveformTables(bool isNewSID)
 #endif
 }
 
+// Here set everything that depends on clock speed and frequency.
+// The player interface imports this.
+void sidEmuConfigureClock( int clockSpeed )
+{
+	sidEmuSetClockSpeed( clockSpeed );
+    
+	PCMsid = (udword)(PCMfreq * (16777216.0 / C64_fClockSpeed));
+	PCMsidNoise = (udword)((C64_fClockSpeed*256.0)/PCMfreq);
+
+    sidEmuChangeReplayingSpeed();
+	sampleEmuInit();
+}
 
 void sidEmuConfigure(udword PCMfrequency, bool measuredEnveValues, 
 					 bool isNewSID, bool emulateFilter, int clockSpeed)
 {
-	sidEmuSetClockSpeed(clockSpeed);  // set clock speed
-
 	PCMfreq = PCMfrequency;
-	PCMsid = (udword)(PCMfrequency * (16777216.0 / C64_fClockSpeed));
-	PCMsidNoise = (udword)((C64_fClockSpeed*256.0)/PCMfrequency);
-
-	sidEmuChangeReplayingSpeed();  // depends on frequency
-	sampleEmuInit();               // depends on clock speed + frequency
-	
+	sidEmuConfigureClock( clockSpeed );
+    
 	filterEnabled = emulateFilter;
 	initWaveformTables(isNewSID);
 	
@@ -1251,7 +1258,6 @@ bool sidEmuReset()
 	toFill = 0;
 #if defined(SIDEMU_TIME_COUNT)
 	prevBufferLen = (scaledBufferLen = 0);
-    bytesCountSong = 0;
 #endif
 	
 	return true;
