@@ -28,7 +28,7 @@ emuEngine::emuEngine()
 	
 #if defined(SIDEMU_TIME_COUNT)
 	// Reset data counter.
-	bytesCount = 0;
+	bytesCountTotal = (bytesCountSong = 0);
 	secondsTotal = (secondsThisSong = 0);
 #endif
 	
@@ -362,20 +362,20 @@ bool emuEngine::allocMem()
 	// Seems as if both tables are needed for panning-mixing with 16-bit samples.
 	// 8-bit
 #ifdef SID_HAVE_EXCEPTIONS
-	if (( ampMod1x8 = new(nothrow) sbyte[256*256] ) == 0 )
+	if (( ampMod1x8 = new(std::nothrow) sbyte[256*256] ) == 0 )
 #else
 	if (( ampMod1x8 = new sbyte[256*256] ) == 0 )
 #endif
 		wasSuccess = false;
 #ifdef SID_HAVE_EXCEPTIONS
-	if (( signedPanMix8 = new(nothrow) sbyte[256*256] ) == 0 )
+	if (( signedPanMix8 = new(std::nothrow) sbyte[256*256] ) == 0 )
 #else
 	if (( signedPanMix8 = new sbyte[256*256] ) == 0 )
 #endif
 		wasSuccess = false;
 	// 16-bit
 #ifdef SID_HAVE_EXCEPTIONS
-	if (( signedPanMix16 = new(nothrow) sword[256*256] ) == 0 )
+	if (( signedPanMix16 = new(std::nothrow) sword[256*256] ) == 0 )
 #else
 	if (( signedPanMix16 = new sword[256*256] ) == 0 )
 #endif
