@@ -10,8 +10,8 @@
 #ifdef SID_HAVE_EXCEPTIONS
 #include <new>
 #endif
-#include <math.h>
-#include <time.h>
+#include <cmath>
+#include <ctime>
 
 #include "emucfg.h"
 #include "6510_.h"
