@@ -76,7 +76,7 @@ bool sidTune::MUS_fileSupport(const void* musBuffer, udword musLength)
                 infoString[line][si++] = c;  // copy to info string
             }
             // If character is 0x9d (left arrow key) then move back.
-            if ((*spPet == 0x9d) && (si>=0))
+            if ((*spPet == 0x9d) && (si>0))
             {
                 si--;
             }
