@@ -6,14 +6,7 @@
 #define SIDPLAY1_MUS__H
 
 
-#include <iostream.h>
-#include <iomanip.h>
 #include "compconf.h"
-#if defined(SID_HAVE_STRSTREA_H)
-#include <strstrea.h>
-#else
-#include <strstream.h>
-#endif
 #include "mytypes.h"
 #include "myendian.h"
 #include "smart.h"
