@@ -6,7 +6,8 @@
 #define SIDPLAY1_PP_H
 
 
-#include <fstream.h>
+#include <fstream>
+using std::ifstream;
 #include "mytypes.h"
 
 extern bool depp(ifstream& inputFile, ubyte** destBufRef);
