@@ -10,16 +10,20 @@
 #ifdef SID_HAVE_EXCEPTIONS
 #include <new>
 #endif
-#include <fstream.h>
-#include <iostream.h>
-#include <iomanip.h>
-#if defined(SID_HAVE_STRSTREA_H)
-  #include <strstrea.h>
+#include <fstream>
+#include <iostream>
+#include <iomanip>
+using namespace std;
+#ifdef SID_HAVE_SSTREAM
+#include <sstream>
+using std::istringstream;
 #else
-  #include <strstream.h>
+#include <strstream.h>
 #endif
-#include <ctype.h>
-#include <string.h>
+#include <string>
+using std::string;
+#include <cctype>
+#include <cstring>
 #include "mytypes.h"
 #include "myendian.h"
 #include "fformat.h"
