@@ -8,10 +8,27 @@
 #ifdef SID_HAVE_EXCEPTIONS
 #include <new>
 #endif
-#include <iostream.h>
-#include <iomanip.h>
-#include <string.h>
-#include <limits.h>
+#include <iostream>
+#include <iomanip>
+#include <fstream>
+using namespace std;
+#include <cstring>
+#include <climits>
+
+#undef SID_USE_STAT
+#ifdef SID_HAVE_UNISTD_H
+#ifdef SID_HAVE_SYS_TYPES_H
+#ifdef SID_HAVE_SYS_STAT_H
+#define SID_USE_STAT
+# include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#endif
+#endif
+#endif
+#ifndef SID_USE_STAT
+#include <stdio.h>
+#endif
 
 #include "sidtune.h"
 #include "fformat.h"
@@ -31,6 +48,8 @@ const char text_dataTooLong[] = "ERROR: Music data size exceeds C64 memory";
 const char text_cantCreateFile[] = "ERROR: Could not create output file";
 const char text_fileIoError[] = "ERROR: File I/O error";
 const char text_fatalInternal[] = "FATAL: Internal error - contact the developers";
+const char text_VBI[] = "VBI";
+const char text_CIA[] = "CIA 1 Timer A";
 const char text_PAL_VBI[] = "50 Hz VBI (PAL)";
 const char text_PAL_CIA[] = "CIA 1 Timer A (PAL)";
 const char text_NTSC_VBI[] = "60 Hz VBI (NTSC)";
@@ -201,30 +220,19 @@ uword sidTune::selectSong(uword selectedSong)
 	info.lengthInSeconds = songLength[selectedSong-1];
 	// Retrieve song speed definition.
 	info.songSpeed = songSpeed[selectedSong-1];
+    // NB! Prior to song initialization, the following is the same
+    // than info.clock.
 	info.clockSpeed = clockSpeed[selectedSong-1];
 	// Assign song speed description string depending on clock speed.
-	if (info.clockSpeed == SIDTUNE_CLOCK_PAL)
-	{
-		if (info.songSpeed == SIDTUNE_SPEED_VBI)
-		{
-			info.speedString = text_PAL_VBI;
-		}
-		else
-		{
-			info.speedString = text_PAL_CIA;
-		}
-	}
-	else  //if (info.clockSpeed == SIDTUNE_CLOCK_NTSC)
-	{
-		if (info.songSpeed == SIDTUNE_SPEED_VBI)
-		{
-			info.speedString = text_NTSC_VBI;
-		}
-		else
-		{
-			info.speedString = text_NTSC_CIA;
-		}
-	}
+    // Final speed description is available only after song init.
+    if (info.songSpeed == SIDTUNE_SPEED_VBI)
+    {
+        info.speedString = text_VBI;
+    }
+    else
+    {
+        info.speedString = text_CIA;
+    }
 	return (info.currentSong=selectedSong);
 }
 
@@ -244,8 +252,7 @@ void sidTune::fixLoadAddress(bool force, uword init, uword play)
     }
 }
 
-
-// ------------------------------------------------- private member functions
+// --------------------------------------------------------------------------
 
 void sidTune::setIRQaddress(uword address)
 {
@@ -292,14 +299,20 @@ udword sidTune::loadFile(const char* fileName, ubyte** bufferRef)
 {
 	udword fileLen = 0;
 	status = false;
-	// Open binary input file stream at end of file.
+    // Work-around for bugs in Standard library.
+    if ( !fileExists( fileName )  )
+    {
+		info.statusString = text_cantOpenFile;
+        return 0;
+    }
+    // Open binary input file stream at end of file.
 #if defined(SID_HAVE_IOS_BIN)
-	ifstream myIn( fileName, ios::in | ios::bin | ios::ate | ios::nocreate );
+	ifstream myIn( fileName, ios::in|ios::bin|ios::ate );
 #else
-	ifstream myIn( fileName, ios::in | ios::binary | ios::ate | ios::nocreate );
+	ifstream myIn( fileName, ios::in|ios::binary|ios::ate );
 #endif
-	// As a replacement for !is_open(), bad() and the NOT-operator don't seem
-	// to work on all systems.
+	// As a replacement for !is_open(), bad() and the NOT-operator
+    // don't seem to work on all systems.
 #if defined(SID_DONT_HAVE_IS_OPEN)
     if ( !myIn )
 #else
@@ -307,81 +320,96 @@ udword sidTune::loadFile(const char* fileName, ubyte** bufferRef)
 #endif
 	{
 		info.statusString = text_cantOpenFile;
+        return 0;
 	}
-	else
-	{
-		// Check for PowerPacker compression: load and decompress, if PP20 file.
-		if (depp(myIn,bufferRef))
-		{
-			// Decompression successful, use uncompressed datafilelen.
-			fileLen = ppUncompressedLen();
-			info.statusString = ppErrorString;
-			status = true;
-		}
-		else if (ppIsCompressed())
-		{
-			// An error occured while decompressing.
-			info.statusString = ppErrorString;
-		}
-		else  // should be uncompressed file
-		{
+    // Check for PowerPacker compression: load and decompress, if PP20 file.
+    if (depp(myIn,bufferRef))
+    {
+        // Decompression successful, use uncompressed datafilelen.
+        fileLen = ppUncompressedLen();
+        info.statusString = ppErrorString;
+        status = true;
+        return fileLen;
+    }
+    else if (ppIsCompressed())
+    {
+        // An error occured while decompressing.
+        info.statusString = ppErrorString;
+        return 0;
+    }
+    // Should be uncompressed file.
 #if defined(SID_HAVE_SEEKG_OFFSET)
-			fileLen = (myIn.seekg(0,ios::end)).offset();
+    fileLen = (myIn.seekg(0,ios::end)).offset();
 #else
-			myIn.seekg(0, ios::end);
-			fileLen = (udword)myIn.tellg();
+    myIn.seekg(0, ios::end);
+    fileLen = (udword)myIn.tellg();
 #endif
-			if ( *bufferRef != 0 )
-			{
-				delete[] *bufferRef;  // free previously allocated memory
-			}
+    if ( *bufferRef != 0 )
+    {
+        delete[] *bufferRef;  // free previously allocated memory
+    }
 #ifdef SID_HAVE_EXCEPTIONS
-			*bufferRef = new(nothrow) ubyte[fileLen+1];
+    *bufferRef = new(std::nothrow) ubyte[fileLen+1];
 #else
-			*bufferRef = new ubyte[fileLen+1];
+    *bufferRef = new ubyte[fileLen+1];
 #endif
-			if ( *bufferRef == 0 )
-			{
-				info.statusString = text_notEnoughMemory;
-				fileLen = 0;  // returning 0 = error condition.
-			}
-			else
-			{
-				*(*bufferRef+fileLen) = 0;
-			}
-			// Load uncompressed file.
-			myIn.seekg(0, ios::beg);
-			udword restFileLen = fileLen;
-			while ( restFileLen > INT_MAX )
-			{
-				myIn.read( (char*)*bufferRef + (fileLen - restFileLen), INT_MAX );
-				restFileLen -= INT_MAX;
-			}
-			if ( restFileLen > 0 )
-			{
-				myIn.read( (char*)*bufferRef + (fileLen - restFileLen), restFileLen );
-			}
-			if ( myIn.bad() )
-			{
-				info.statusString = text_cantLoadFile;
-			}
-			else
-			{
-				info.statusString = text_noErrors;
-				status = true;
-			}
-		}
-		myIn.close();
-        if (fileLen == 0)
-        {
-            info.statusString = text_emptyFile;
-            status = false;
-        }
-	}
+    if ( *bufferRef == 0 )
+    {
+        info.statusString = text_notEnoughMemory;
+        fileLen = 0;  // returning 0 = error condition.
+    }
+    else
+    {
+        *(*bufferRef+fileLen) = 0;
+    }
+    // Load uncompressed file.
+    myIn.seekg(0, ios::beg);
+    udword restFileLen = fileLen;
+    while ( restFileLen > INT_MAX )
+    {
+        myIn.read( (char*)*bufferRef + (fileLen - restFileLen), INT_MAX );
+        restFileLen -= INT_MAX;
+    }
+    if ( restFileLen > 0 )
+    {
+        myIn.read( (char*)*bufferRef + (fileLen - restFileLen), restFileLen );
+    }
+    if ( myIn.bad() )
+    {
+        info.statusString = text_cantLoadFile;
+    }
+    else
+    {
+        info.statusString = text_noErrors;
+        status = true;
+    }
+    myIn.close();
+    if (fileLen == 0)
+    {
+        info.statusString = text_emptyFile;
+        status = false;
+    }
 	return fileLen;
 }
 
 
+// protected
+bool sidTune::fileExists( const char* fileName )
+{
+#ifdef SID_USE_STAT
+    struct stat fileStat;
+    return ( stat( fileName, &fileStat ) == 0  &&
+         S_ISREG( fileStat.st_mode )  );
+#else
+    FILE* f = fopen( fileName, "r+b" );
+    bool exists = (f!=NULL);
+    if ( exists )
+        fclose( f );
+    return exists;
+#endif
+}
+    
+
 void sidTune::deleteFileBuffers()
 {
 	// This function does not affect status and statusstring.
@@ -417,7 +445,7 @@ bool sidTune::cacheRawData(const void* sourceBuf, udword sourceBufLen)
 {
 	clearCache();
 #ifdef SID_HAVE_EXCEPTIONS
-	if ( (cachePtr = new(nothrow) ubyte[sourceBufLen]) == 0 )
+	if ( (cachePtr = new(std::nothrow) ubyte[sourceBufLen]) == 0 )
 #else
 	if ( (cachePtr = new ubyte[sourceBufLen]) == 0 )
 #endif
@@ -480,15 +508,21 @@ void sidTune::safeConstructor()
 	info.loadAddr = ( info.initAddr = ( info.playAddr = 0 ));
 	info.songs = ( info.startSong = ( info.currentSong = 0 ));
 	info.musPlayer = false;
+	info.psidSpecific = false;
 	info.fixLoad = false;
+    info.clock = SIDTUNE_CLOCK_UNKNOWN;
+    info.sidModel = SIDTUNE_SIDMODEL_UNKNOWN;
 	info.songSpeed = SIDTUNE_SPEED_VBI;
-	info.clockSpeed = SIDTUNE_CLOCK_PAL;
+	info.clockSpeed = SIDTUNE_CLOCK_UNKNOWN;
 	info.lengthInSeconds = 0;
+    info.relocStartPage = 0;
+    info.relocPages = 0;
+    info.reserved = 0;
 	
 	for ( uint si = 0; si < classMaxSongs; si++ )
 	{
 		songSpeed[si] = SIDTUNE_SPEED_VBI;
-		clockSpeed[si] = SIDTUNE_CLOCK_PAL;
+		clockSpeed[si] = SIDTUNE_CLOCK_UNKNOWN;
 		songLength[si] = 0;
 	}
 
@@ -511,7 +545,7 @@ void sidTune::safeConstructor()
 	// Not used!!!
 	info.numberOfCommentStrings = 1;
 #ifdef SID_HAVE_EXCEPTIONS
-	info.commentString = new(nothrow) char* [info.numberOfCommentStrings];
+	info.commentString = new(std::nothrow) char* [info.numberOfCommentStrings];
 #else
 	info.commentString = new char* [info.numberOfCommentStrings];
 #endif
@@ -555,7 +589,7 @@ void sidTune::stdinConstructor()
 	// Assume the memory allocation to fail.
 	info.statusString = text_notEnoughMemory;
 #ifdef SID_HAVE_EXCEPTIONS
-	if (( fileBuf = new(nothrow) ubyte[maxSidtuneFileLen] ) == 0 )
+	if (( fileBuf = new(std::nothrow) ubyte[maxSidtuneFileLen] ) == 0 )
 #else
 	if (( fileBuf = new ubyte[maxSidtuneFileLen] ) == 0 )
 #endif
@@ -563,7 +597,7 @@ void sidTune::stdinConstructor()
 		return;
     }
 	udword i = 0;
-	ubyte datb;
+	char datb;
 	// We only read as much as fits in the buffer.
 	// This way we avoid choking on huge data.
 	while (cin.get(datb) && i<maxSidtuneFileLen)
@@ -689,7 +723,7 @@ bool sidTune::createNewFileName( char** destStringPtr,
 	}
 	// Get enough memory, so we can appended the extension.
 #ifdef SID_HAVE_EXCEPTIONS
-	*destStringPtr = new(nothrow) char[strlen(sourceName) + strlen(sourceExt) +1];
+	*destStringPtr = new(std::nothrow) char[strlen(sourceName) + strlen(sourceExt) +1];
 #else
 	*destStringPtr = new char[strlen(sourceName) + strlen(sourceExt) +1];
 #endif
@@ -758,7 +792,7 @@ void sidTune::filesConstructor( const char* fileName )
 					if ( !createNewFileName(&fileName2,fileName,*tmpFileNameExt) )
 						return;
 					// Do not load the first file again if names are equal.
-					if ( stricmp(fileName,fileName2) != 0 )
+					if ( myStrCaseCmp(fileName,fileName2) != 0 )
 					{
 						// 1st data file was loaded into ``fileBuf'',
 						// so we load the 2nd one into ``fileBuf2''.
@@ -812,7 +846,7 @@ void sidTune::filesConstructor( const char* fileName )
 					if ( !createNewFileName(&fileName2,fileName,*tmpFileNameExt) )
 						return;
 					// Do not load the first file again if names are equal.
-					if ( stricmp(fileName,fileName2) != 0 )
+					if ( myStrCaseCmp(fileName,fileName2) != 0 )
 					{
 						// 1st info file was loaded into ``fileBuf'',
 						// so we load the 2nd one into ``fileBuf2''.
@@ -887,13 +921,12 @@ void sidTune::convertOldStyleSpeedToTables(udword oldStyleSpeed)
 		if (( (oldStyleSpeed>>(s&31)) & 1 ) == 0 )
 		{
 			songSpeed[s] = SIDTUNE_SPEED_VBI;
-			clockSpeed[s] = SIDTUNE_CLOCK_PAL;
 		}
 		else
 		{
 			songSpeed[s] = SIDTUNE_SPEED_CIA_1A;
-			clockSpeed[s] = SIDTUNE_CLOCK_PAL;
 		}
+        clockSpeed[s] = info.clock;
 	}
 }
 
@@ -931,18 +964,19 @@ bool sidTune::saveC64dataFile( const char* fileName, bool overWriteFlag )
 	// This prevents saving from a bad object.
 	if ( status )
 	{
+		ofstream fMyOut;
+        if ( !overWriteFlag && fileExists( fileName ) )
+        {
+			info.statusString = text_cantCreateFile;
+            return success;
+        }
 		// Open binary output file stream.
-		long int createAttr;
+        else
 #if defined(SID_HAVE_IOS_BIN)
-		createAttr = ios::out | ios::bin;
+            fMyOut.open( fileName, ios::out|ios::bin|ios::trunc );
 #else
-		createAttr = ios::out | ios::binary;
+            fMyOut.open( fileName, ios::out|ios::binary|ios::trunc );
 #endif
-		if ( overWriteFlag )
-			createAttr |= ios::trunc;
-		else
-			createAttr |= ios::noreplace;
-		ofstream fMyOut( fileName, createAttr );
 		if ( !fMyOut )
 		{ 
 			info.statusString = text_cantCreateFile;
@@ -978,14 +1012,19 @@ bool sidTune::saveSIDfile( const char* fileName, bool overWriteFlag )
 	// This prevents saving from a bad object.
 	if ( status )
 	{
+		ofstream fMyOut;
+        if ( !overWriteFlag && fileExists( fileName ) )
+        {
+			info.statusString = text_cantCreateFile;
+            return success;
+        }
 		// Open ASCII output file stream.
-		long int createAttr;
-		createAttr = ios::out;
-		if ( overWriteFlag )
-			createAttr |= ios::trunc;
-		else
-			createAttr |= ios::noreplace;
-		ofstream fMyOut( fileName, createAttr );
+        else
+#if defined(SID_HAVE_IOS_BIN)
+            fMyOut.open( fileName, ios::out|ios::trunc );
+#else
+            fMyOut.open( fileName, ios::out|ios::trunc );
+#endif
 		if ( !fMyOut )
 		{ 
 			info.statusString = text_cantCreateFile;
@@ -1014,18 +1053,19 @@ bool sidTune::savePSIDfile( const char* fileName, bool overWriteFlag )
 	// This prevents saving from a bad object.
 	if ( status )
 	{
+		ofstream fMyOut;
+        if ( !overWriteFlag && fileExists( fileName ) )
+        {
+			info.statusString = text_cantCreateFile;
+            return success;
+        }
 		// Open binary output file stream.
-		long int createAttr;
+        else
 #if defined(SID_HAVE_IOS_BIN)
-		createAttr = ios::out | ios::bin;
+            fMyOut.open( fileName, ios::out|ios::bin|ios::trunc );
 #else
-		createAttr = ios::out | ios::binary;
+            fMyOut.open( fileName, ios::out|ios::binary|ios::trunc );
 #endif
-	  if ( overWriteFlag )
-			createAttr |= ios::trunc;
-		else
-			createAttr |= ios::noreplace;
-		ofstream fMyOut( fileName, createAttr );
 		if ( !fMyOut )
 		{
 			info.statusString = text_cantCreateFile;
